@@ -14,6 +14,7 @@
 package azure
 
 import (
+	"context"
 	"fmt"
 	"net"
 	"strings"
@@ -23,13 +24,15 @@ import (
 	"github.com/Azure/azure-sdk-for-go/arm/network"
 	"github.com/Azure/go-autorest/autorest/azure"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
+	config_util "github.com/prometheus/common/config"
 	"github.com/prometheus/common/model"
-	"golang.org/x/net/context"
 
-	"github.com/prometheus/prometheus/config"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
 	"github.com/prometheus/prometheus/util/strutil"
+	yaml_util "github.com/prometheus/prometheus/util/yaml"
 )
 
 const (
@@ -53,24 +56,58 @@ var (
 			Name: "prometheus_sd_azure_refresh_duration_seconds",
 			Help: "The duration of a Azure-SD refresh in seconds.",
 		})
+
+	// DefaultSDConfig is the default Azure SD configuration.
+	DefaultSDConfig = SDConfig{
+		Port:            80,
+		RefreshInterval: model.Duration(5 * time.Minute),
+	}
 )
 
+// SDConfig is the configuration for Azure based service discovery.
+type SDConfig struct {
+	Port            int                `yaml:"port"`
+	SubscriptionID  string             `yaml:"subscription_id"`
+	TenantID        string             `yaml:"tenant_id,omitempty"`
+	ClientID        string             `yaml:"client_id,omitempty"`
+	ClientSecret    config_util.Secret `yaml:"client_secret,omitempty"`
+	RefreshInterval model.Duration     `yaml:"refresh_interval,omitempty"`
+
+	// Catches all undefined fields and must be empty after parsing.
+	XXX map[string]interface{} `yaml:",inline"`
+}
+
+// UnmarshalYAML implements the yaml.Unmarshaler interface.
+func (c *SDConfig) UnmarshalYAML(unmarshal func(interface{}) error) error {
+	*c = DefaultSDConfig
+	type plain SDConfig
+	err := unmarshal((*plain)(c))
+	if err != nil {
+		return err
+	}
+
+	return yaml_util.CheckOverflow(c.XXX, "azure_sd_config")
+}
+
 func init() {
 	prometheus.MustRegister(azureSDRefreshDuration)
 	prometheus.MustRegister(azureSDRefreshFailuresCount)
 }
 
 // Discovery periodically performs Azure-SD requests. It implements
-// the TargetProvider interface.
+// the Discoverer interface.
 type Discovery struct {
-	cfg      *config.AzureSDConfig
+	cfg      *SDConfig
 	interval time.Duration
 	port     int
 	logger   log.Logger
 }
 
 // NewDiscovery returns a new AzureDiscovery which periodically refreshes its targets.
-func NewDiscovery(cfg *config.AzureSDConfig, logger log.Logger) *Discovery {
+func NewDiscovery(cfg *SDConfig, logger log.Logger) *Discovery {
+	if logger == nil {
+		logger = log.NewNopLogger()
+	}
 	return &Discovery{
 		cfg:      cfg,
 		interval: time.Duration(cfg.RefreshInterval),
@@ -79,8 +116,8 @@ func NewDiscovery(cfg *config.AzureSDConfig, logger log.Logger) *Discovery {
 	}
 }
 
-// Run implements the TargetProvider interface.
-func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
+// Run implements the Discoverer interface.
+func (d *Discovery) Run(ctx context.Context, ch chan<- []*targetgroup.Group) {
 	ticker := time.NewTicker(d.interval)
 	defer ticker.Stop()
 
@@ -93,11 +130,11 @@ func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 
 		tg, err := d.refresh()
 		if err != nil {
-			d.logger.Errorf("unable to refresh during Azure discovery: %s", err)
+			level.Error(d.logger).Log("msg", "Unable to refresh during Azure discovery", "err", err)
 		} else {
 			select {
 			case <-ctx.Done():
-			case ch <- []*config.TargetGroup{tg}:
+			case ch <- []*targetgroup.Group{tg}:
 			}
 		}
 
@@ -116,7 +153,7 @@ type azureClient struct {
 }
 
 // createAzureClient is a helper function for creating an Azure compute client to ARM.
-func createAzureClient(cfg config.AzureSDConfig) (azureClient, error) {
+func createAzureClient(cfg SDConfig) (azureClient, error) {
 	var c azureClient
 	oauthConfig, err := azure.PublicCloud.OAuthConfigForTenant(cfg.TenantID)
 	if err != nil {
@@ -149,7 +186,7 @@ func newAzureResourceFromID(id string, logger log.Logger) (azureResource, error)
 	s := strings.Split(id, "/")
 	if len(s) != 9 {
 		err := fmt.Errorf("invalid ID '%s'. Refusing to create azureResource", id)
-		logger.Error(err)
+		level.Error(logger).Log("err", err)
 		return azureResource{}, err
 	}
 	return azureResource{
@@ -158,7 +195,9 @@ func newAzureResourceFromID(id string, logger log.Logger) (azureResource, error)
 	}, nil
 }
 
-func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
+func (d *Discovery) refresh() (tg *targetgroup.Group, err error) {
+	defer level.Debug(d.logger).Log("msg", "Azure discovery completed")
+
 	t0 := time.Now()
 	defer func() {
 		azureSDRefreshDuration.Observe(time.Since(t0).Seconds())
@@ -166,7 +205,7 @@ func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
 			azureSDRefreshFailuresCount.Inc()
 		}
 	}()
-	tg = &config.TargetGroup{}
+	tg = &targetgroup.Group{}
 	client, err := createAzureClient(*d.cfg)
 	if err != nil {
 		return tg, fmt.Errorf("could not create Azure client: %s", err)
@@ -187,7 +226,7 @@ func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
 		}
 		machines = append(machines, *result.Value...)
 	}
-	d.logger.Debugf("Found %d virtual machines during Azure discovery.", len(machines))
+	level.Debug(d.logger).Log("msg", "Found virtual machines during Azure discovery.", "count", len(machines))
 
 	// We have the slice of machines. Now turn them into targets.
 	// Doing them in go routines because the network interface calls are slow.
@@ -228,7 +267,7 @@ func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
 				}
 				networkInterface, err := client.nic.Get(r.ResourceGroup, r.Name, "")
 				if err != nil {
-					d.logger.Errorf("Unable to get network interface %s: %s", r.Name, err)
+					level.Error(d.logger).Log("msg", "Unable to get network interface", "name", r.Name, "err", err)
 					ch <- target{labelSet: nil, err: err}
 					// Get out of this routine because we cannot continue without a network interface.
 					return
@@ -239,7 +278,7 @@ func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
 				// yet support this. On deallocated machines, this value happens to be nil so it
 				// is a cheap and easy way to determine if a machine is allocated or not.
 				if networkInterface.Properties.Primary == nil {
-					d.logger.Debugf("Virtual machine %s is deallocated. Skipping during Azure SD.", *vm.Name)
+					level.Debug(d.logger).Log("msg", "Skipping deallocated virtual machine", "machine", *vm.Name)
 					ch <- target{}
 					return
 				}
@@ -274,6 +313,5 @@ func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
 		}
 	}
 
-	d.logger.Debugf("Azure discovery completed.")
 	return tg, nil
 }
