@@ -16,9 +16,8 @@ package kubernetes
 import (
 	"testing"
 
-	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/config"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/client-go/pkg/apis/extensions/v1beta1"
 )
@@ -33,7 +32,7 @@ func newFakeIngressInformer() *fakeInformer {
 
 func makeTestIngressDiscovery() (*Ingress, *fakeInformer) {
 	i := newFakeIngressInformer()
-	return NewIngress(log.Base(), i), i
+	return NewIngress(nil, i), i
 }
 
 func makeIngress(tls []v1beta1.IngressTLS) *v1beta1.Ingress {
@@ -78,12 +77,12 @@ func makeIngress(tls []v1beta1.IngressTLS) *v1beta1.Ingress {
 	}
 }
 
-func expectedTargetGroups(tls bool) []*config.TargetGroup {
+func expectedTargetGroups(tls bool) []*targetgroup.Group {
 	scheme := "http"
 	if tls {
 		scheme = "https"
 	}
-	return []*config.TargetGroup{
+	return []*targetgroup.Group{
 		{
 			Targets: []model.LabelSet{
 				{
