@@ -14,15 +14,16 @@
 package kubernetes
 
 import (
+	"context"
 	"fmt"
 	"net"
 	"strconv"
 
-	"github.com/prometheus/common/log"
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/config"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
 	"github.com/prometheus/prometheus/util/strutil"
-	"golang.org/x/net/context"
 	"k8s.io/client-go/pkg/api"
 	apiv1 "k8s.io/client-go/pkg/api/v1"
 	"k8s.io/client-go/tools/cache"
@@ -37,13 +38,16 @@ type Node struct {
 
 // NewNode returns a new node discovery.
 func NewNode(l log.Logger, inf cache.SharedInformer) *Node {
+	if l == nil {
+		l = log.NewNopLogger()
+	}
 	return &Node{logger: l, informer: inf, store: inf.GetStore()}
 }
 
-// Run implements the TargetProvider interface.
-func (n *Node) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
+// Run implements the Discoverer interface.
+func (n *Node) Run(ctx context.Context, ch chan<- []*targetgroup.Group) {
 	// Send full initial set of pod targets.
-	var initial []*config.TargetGroup
+	var initial []*targetgroup.Group
 	for _, o := range n.store.List() {
 		tg := n.buildNode(o.(*apiv1.Node))
 		initial = append(initial, tg)
@@ -55,10 +59,10 @@ func (n *Node) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 	}
 
 	// Send target groups for service updates.
-	send := func(tg *config.TargetGroup) {
+	send := func(tg *targetgroup.Group) {
 		select {
 		case <-ctx.Done():
-		case ch <- []*config.TargetGroup{tg}:
+		case ch <- []*targetgroup.Group{tg}:
 		}
 	}
 	n.informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
@@ -67,7 +71,7 @@ func (n *Node) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 
 			node, err := convertToNode(o)
 			if err != nil {
-				n.logger.With("err", err).Errorln("converting to Node object failed")
+				level.Error(n.logger).Log("msg", "converting to Node object failed", "err", err)
 				return
 			}
 			send(n.buildNode(node))
@@ -77,17 +81,17 @@ func (n *Node) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 
 			node, err := convertToNode(o)
 			if err != nil {
-				n.logger.With("err", err).Errorln("converting to Node object failed")
+				level.Error(n.logger).Log("msg", "converting to Node object failed", "err", err)
 				return
 			}
-			send(&config.TargetGroup{Source: nodeSource(node)})
+			send(&targetgroup.Group{Source: nodeSource(node)})
 		},
 		UpdateFunc: func(_, o interface{}) {
 			eventCount.WithLabelValues("node", "update").Inc()
 
 			node, err := convertToNode(o)
 			if err != nil {
-				n.logger.With("err", err).Errorln("converting to Node object failed")
+				level.Error(n.logger).Log("msg", "converting to Node object failed", "err", err)
 				return
 			}
 			send(n.buildNode(node))
@@ -143,15 +147,15 @@ func nodeLabels(n *apiv1.Node) model.LabelSet {
 	return ls
 }
 
-func (n *Node) buildNode(node *apiv1.Node) *config.TargetGroup {
-	tg := &config.TargetGroup{
+func (n *Node) buildNode(node *apiv1.Node) *targetgroup.Group {
+	tg := &targetgroup.Group{
 		Source: nodeSource(node),
 	}
 	tg.Labels = nodeLabels(node)
 
 	addr, addrMap, err := nodeAddress(node)
 	if err != nil {
-		n.logger.With("err", err).Debugf("No node address found")
+		level.Warn(n.logger).Log("msg", "No node address found", "err", err)
 		return nil
 	}
 	addr = net.JoinHostPort(addr, strconv.FormatInt(int64(node.Status.DaemonEndpoints.KubeletEndpoint.Port), 10))
