@@ -14,17 +14,16 @@
 package kubernetes
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"sync"
 	"testing"
 	"time"
 
-	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/config"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
 	"github.com/stretchr/testify/require"
-	"golang.org/x/net/context"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/client-go/pkg/api/v1"
 	"k8s.io/client-go/tools/cache"
@@ -104,19 +103,19 @@ func (i *fakeInformer) Update(obj interface{}) {
 	}
 }
 
-type targetProvider interface {
-	Run(ctx context.Context, up chan<- []*config.TargetGroup)
+type discoverer interface {
+	Run(ctx context.Context, up chan<- []*targetgroup.Group)
 }
 
 type k8sDiscoveryTest struct {
-	discovery       targetProvider
+	discovery       discoverer
 	afterStart      func()
-	expectedInitial []*config.TargetGroup
-	expectedRes     []*config.TargetGroup
+	expectedInitial []*targetgroup.Group
+	expectedRes     []*targetgroup.Group
 }
 
 func (d k8sDiscoveryTest) Run(t *testing.T) {
-	ch := make(chan []*config.TargetGroup)
+	ch := make(chan []*targetgroup.Group)
 	ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*10)
 	defer cancel()
 	go func() {
@@ -136,7 +135,7 @@ func (d k8sDiscoveryTest) Run(t *testing.T) {
 	}
 }
 
-func requireTargetGroups(t *testing.T, expected, res []*config.TargetGroup) {
+func requireTargetGroups(t *testing.T, expected, res []*targetgroup.Group) {
 	b1, err := json.Marshal(expected)
 	if err != nil {
 		panic(err)
@@ -159,7 +158,7 @@ func newFakeNodeInformer() *fakeInformer {
 
 func makeTestNodeDiscovery() (*Node, *fakeInformer) {
 	i := newFakeNodeInformer()
-	return NewNode(log.Base(), i), i
+	return NewNode(nil, i), i
 }
 
 func makeNode(name, address string, labels map[string]string, annotations map[string]string) *v1.Node {
@@ -200,7 +199,7 @@ func TestNodeDiscoveryInitial(t *testing.T) {
 
 	k8sDiscoveryTest{
 		discovery: n,
-		expectedInitial: []*config.TargetGroup{
+		expectedInitial: []*targetgroup.Group{
 			{
 				Targets: []model.LabelSet{
 					{
@@ -226,7 +225,7 @@ func TestNodeDiscoveryAdd(t *testing.T) {
 	k8sDiscoveryTest{
 		discovery:  n,
 		afterStart: func() { go func() { i.Add(makeEnumeratedNode(1)) }() },
-		expectedRes: []*config.TargetGroup{
+		expectedRes: []*targetgroup.Group{
 			{
 				Targets: []model.LabelSet{
 					{
@@ -251,7 +250,7 @@ func TestNodeDiscoveryDelete(t *testing.T) {
 	k8sDiscoveryTest{
 		discovery:  n,
 		afterStart: func() { go func() { i.Delete(makeEnumeratedNode(0)) }() },
-		expectedInitial: []*config.TargetGroup{
+		expectedInitial: []*targetgroup.Group{
 			{
 				Targets: []model.LabelSet{
 					{
@@ -266,7 +265,7 @@ func TestNodeDiscoveryDelete(t *testing.T) {
 				Source: "node/test0",
 			},
 		},
-		expectedRes: []*config.TargetGroup{
+		expectedRes: []*targetgroup.Group{
 			{
 				Source: "node/test0",
 			},
@@ -281,7 +280,7 @@ func TestNodeDiscoveryDeleteUnknownCacheState(t *testing.T) {
 	k8sDiscoveryTest{
 		discovery:  n,
 		afterStart: func() { go func() { i.Delete(cache.DeletedFinalStateUnknown{Obj: makeEnumeratedNode(0)}) }() },
-		expectedInitial: []*config.TargetGroup{
+		expectedInitial: []*targetgroup.Group{
 			{
 				Targets: []model.LabelSet{
 					{
@@ -296,7 +295,7 @@ func TestNodeDiscoveryDeleteUnknownCacheState(t *testing.T) {
 				Source: "node/test0",
 			},
 		},
-		expectedRes: []*config.TargetGroup{
+		expectedRes: []*targetgroup.Group{
 			{
 				Source: "node/test0",
 			},
@@ -322,7 +321,7 @@ func TestNodeDiscoveryUpdate(t *testing.T) {
 				)
 			}()
 		},
-		expectedInitial: []*config.TargetGroup{
+		expectedInitial: []*targetgroup.Group{
 			{
 				Targets: []model.LabelSet{
 					{
@@ -337,7 +336,7 @@ func TestNodeDiscoveryUpdate(t *testing.T) {
 				Source: "node/test0",
 			},
 		},
-		expectedRes: []*config.TargetGroup{
+		expectedRes: []*targetgroup.Group{
 			{
 				Targets: []model.LabelSet{
 					{
