@@ -14,19 +14,21 @@
 package kubernetes
 
 import (
+	"context"
 	"fmt"
 	"net"
 	"strconv"
 	"strings"
 
-	"github.com/prometheus/common/log"
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/config"
-	"github.com/prometheus/prometheus/util/strutil"
-	"golang.org/x/net/context"
 	"k8s.io/client-go/pkg/api"
 	apiv1 "k8s.io/client-go/pkg/api/v1"
 	"k8s.io/client-go/tools/cache"
+
+	"github.com/prometheus/prometheus/discovery/targetgroup"
+	"github.com/prometheus/prometheus/util/strutil"
 )
 
 // Pod discovers new pod targets.
@@ -38,6 +40,9 @@ type Pod struct {
 
 // NewPod creates a new pod discovery.
 func NewPod(l log.Logger, pods cache.SharedInformer) *Pod {
+	if l == nil {
+		l = log.NewNopLogger()
+	}
 	return &Pod{
 		informer: pods,
 		store:    pods.GetStore(),
@@ -45,15 +50,15 @@ func NewPod(l log.Logger, pods cache.SharedInformer) *Pod {
 	}
 }
 
-// Run implements the TargetProvider interface.
-func (p *Pod) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
+// Run implements the Discoverer interface.
+func (p *Pod) Run(ctx context.Context, ch chan<- []*targetgroup.Group) {
 	// Send full initial set of pod targets.
-	var initial []*config.TargetGroup
+	var initial []*targetgroup.Group
 	for _, o := range p.store.List() {
 		tg := p.buildPod(o.(*apiv1.Pod))
 		initial = append(initial, tg)
 
-		p.logger.With("tg", fmt.Sprintf("%#v", tg)).Debugln("initial pod")
+		level.Debug(p.logger).Log("msg", "initial pod", "tg", fmt.Sprintf("%#v", tg))
 	}
 	select {
 	case <-ctx.Done():
@@ -62,11 +67,11 @@ func (p *Pod) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 	}
 
 	// Send target groups for pod updates.
-	send := func(tg *config.TargetGroup) {
-		p.logger.With("tg", fmt.Sprintf("%#v", tg)).Debugln("pod update")
+	send := func(tg *targetgroup.Group) {
+		level.Debug(p.logger).Log("msg", "pod update", "tg", fmt.Sprintf("%#v", tg))
 		select {
 		case <-ctx.Done():
-		case ch <- []*config.TargetGroup{tg}:
+		case ch <- []*targetgroup.Group{tg}:
 		}
 	}
 	p.informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
@@ -75,7 +80,7 @@ func (p *Pod) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 
 			pod, err := convertToPod(o)
 			if err != nil {
-				p.logger.With("err", err).Errorln("converting to Pod object failed")
+				level.Error(p.logger).Log("msg", "converting to Pod object failed", "err", err)
 				return
 			}
 			send(p.buildPod(pod))
@@ -85,17 +90,17 @@ func (p *Pod) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 
 			pod, err := convertToPod(o)
 			if err != nil {
-				p.logger.With("err", err).Errorln("converting to Pod object failed")
+				level.Error(p.logger).Log("msg", "converting to Pod object failed", "err", err)
 				return
 			}
-			send(&config.TargetGroup{Source: podSource(pod)})
+			send(&targetgroup.Group{Source: podSource(pod)})
 		},
 		UpdateFunc: func(_, o interface{}) {
 			eventCount.WithLabelValues("pod", "update").Inc()
 
 			pod, err := convertToPod(o)
 			if err != nil {
-				p.logger.With("err", err).Errorln("converting to Pod object failed")
+				level.Error(p.logger).Log("msg", "converting to Pod object failed", "err", err)
 				return
 			}
 			send(p.buildPod(pod))
@@ -135,6 +140,7 @@ const (
 	podAnnotationPrefix           = metaLabelPrefix + "pod_annotation_"
 	podNodeNameLabel              = metaLabelPrefix + "pod_node_name"
 	podHostIPLabel                = metaLabelPrefix + "pod_host_ip"
+	podUID                        = metaLabelPrefix + "pod_uid"
 )
 
 func podLabels(pod *apiv1.Pod) model.LabelSet {
@@ -144,6 +150,7 @@ func podLabels(pod *apiv1.Pod) model.LabelSet {
 		podReadyLabel:    podReady(pod),
 		podNodeNameLabel: lv(pod.Spec.NodeName),
 		podHostIPLabel:   lv(pod.Status.HostIP),
+		podUID:           lv(string(pod.ObjectMeta.UID)),
 	}
 
 	for k, v := range pod.Labels {
@@ -159,13 +166,13 @@ func podLabels(pod *apiv1.Pod) model.LabelSet {
 	return ls
 }
 
-func (p *Pod) buildPod(pod *apiv1.Pod) *config.TargetGroup {
+func (p *Pod) buildPod(pod *apiv1.Pod) *targetgroup.Group {
 	// During startup the pod may not have an IP yet. This does not even allow
 	// for an up metric, so we skip the target.
 	if len(pod.Status.PodIP) == 0 {
 		return nil
 	}
-	tg := &config.TargetGroup{
+	tg := &targetgroup.Group{
 		Source: podSource(pod),
 	}
 	tg.Labels = podLabels(pod)
