@@ -14,23 +14,27 @@
 package marathon
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"io/ioutil"
 	"math/rand"
 	"net"
 	"net/http"
+	"strconv"
 	"strings"
 	"time"
 
-	"golang.org/x/net/context"
-
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
+	conntrack "github.com/mwitkow/go-conntrack"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
+	config_util "github.com/prometheus/common/config"
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/config"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
 	"github.com/prometheus/prometheus/util/httputil"
 	"github.com/prometheus/prometheus/util/strutil"
+	yaml_util "github.com/prometheus/prometheus/util/yaml"
 )
 
 const (
@@ -43,6 +47,9 @@ const (
 	appLabel model.LabelName = metaLabelPrefix + "app"
 	// imageLabel is the label that is used for the docker image running the service.
 	imageLabel model.LabelName = metaLabelPrefix + "image"
+	// portIndexLabel is the integer port index when multiple ports are defined;
+	// e.g. PORT1 would have a value of '1'
+	portIndexLabel model.LabelName = metaLabelPrefix + "port_index"
 	// taskLabel contains the mesos task name of the app instance.
 	taskLabel model.LabelName = metaLabelPrefix + "task"
 
@@ -68,8 +75,47 @@ var (
 			Name:      "sd_marathon_refresh_duration_seconds",
 			Help:      "The duration of a Marathon-SD refresh in seconds.",
 		})
+	// DefaultSDConfig is the default Marathon SD configuration.
+	DefaultSDConfig = SDConfig{
+		Timeout:         model.Duration(30 * time.Second),
+		RefreshInterval: model.Duration(30 * time.Second),
+	}
 )
 
+// SDConfig is the configuration for services running on Marathon.
+type SDConfig struct {
+	Servers         []string              `yaml:"servers,omitempty"`
+	Timeout         model.Duration        `yaml:"timeout,omitempty"`
+	RefreshInterval model.Duration        `yaml:"refresh_interval,omitempty"`
+	TLSConfig       config_util.TLSConfig `yaml:"tls_config,omitempty"`
+	BearerToken     config_util.Secret    `yaml:"bearer_token,omitempty"`
+	BearerTokenFile string                `yaml:"bearer_token_file,omitempty"`
+
+	// Catches all undefined fields and must be empty after parsing.
+	XXX map[string]interface{} `yaml:",inline"`
+}
+
+// UnmarshalYAML implements the yaml.Unmarshaler interface.
+func (c *SDConfig) UnmarshalYAML(unmarshal func(interface{}) error) error {
+	*c = DefaultSDConfig
+	type plain SDConfig
+	err := unmarshal((*plain)(c))
+	if err != nil {
+		return err
+	}
+	if err := yaml_util.CheckOverflow(c.XXX, "marathon_sd_config"); err != nil {
+		return err
+	}
+	if len(c.Servers) == 0 {
+		return fmt.Errorf("Marathon SD config must contain at least one Marathon server")
+	}
+	if len(c.BearerToken) > 0 && len(c.BearerTokenFile) > 0 {
+		return fmt.Errorf("at most one of bearer_token & bearer_token_file must be configured")
+	}
+
+	return nil
+}
+
 func init() {
 	prometheus.MustRegister(refreshFailuresCount)
 	prometheus.MustRegister(refreshDuration)
@@ -82,14 +128,18 @@ type Discovery struct {
 	client          *http.Client
 	servers         []string
 	refreshInterval time.Duration
-	lastRefresh     map[string]*config.TargetGroup
+	lastRefresh     map[string]*targetgroup.Group
 	appsClient      AppListClient
 	token           string
 	logger          log.Logger
 }
 
 // NewDiscovery returns a new Marathon Discovery.
-func NewDiscovery(conf *config.MarathonSDConfig, logger log.Logger) (*Discovery, error) {
+func NewDiscovery(conf SDConfig, logger log.Logger) (*Discovery, error) {
+	if logger == nil {
+		logger = log.NewNopLogger()
+	}
+
 	tls, err := httputil.NewTLSConfig(conf.TLSConfig)
 	if err != nil {
 		return nil, err
@@ -108,6 +158,10 @@ func NewDiscovery(conf *config.MarathonSDConfig, logger log.Logger) (*Discovery,
 		Timeout: time.Duration(conf.Timeout),
 		Transport: &http.Transport{
 			TLSClientConfig: tls,
+			DialContext: conntrack.NewDialContextFunc(
+				conntrack.DialWithTracing(),
+				conntrack.DialWithName("marathon_sd"),
+			),
 		},
 	}
 
@@ -121,8 +175,8 @@ func NewDiscovery(conf *config.MarathonSDConfig, logger log.Logger) (*Discovery,
 	}, nil
 }
 
-// Run implements the TargetProvider interface.
-func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
+// Run implements the Discoverer interface.
+func (d *Discovery) Run(ctx context.Context, ch chan<- []*targetgroup.Group) {
 	for {
 		select {
 		case <-ctx.Done():
@@ -130,13 +184,13 @@ func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 		case <-time.After(d.refreshInterval):
 			err := d.updateServices(ctx, ch)
 			if err != nil {
-				d.logger.Errorf("Error while updating services: %s", err)
+				level.Error(d.logger).Log("msg", "Error while updating services", "err", err)
 			}
 		}
 	}
 }
 
-func (d *Discovery) updateServices(ctx context.Context, ch chan<- []*config.TargetGroup) (err error) {
+func (d *Discovery) updateServices(ctx context.Context, ch chan<- []*targetgroup.Group) (err error) {
 	t0 := time.Now()
 	defer func() {
 		refreshDuration.Observe(time.Since(t0).Seconds())
@@ -150,7 +204,7 @@ func (d *Discovery) updateServices(ctx context.Context, ch chan<- []*config.Targ
 		return err
 	}
 
-	all := make([]*config.TargetGroup, 0, len(targetMap))
+	all := make([]*targetgroup.Group, 0, len(targetMap))
 	for _, tg := range targetMap {
 		all = append(all, tg)
 	}
@@ -168,8 +222,8 @@ func (d *Discovery) updateServices(ctx context.Context, ch chan<- []*config.Targ
 			select {
 			case <-ctx.Done():
 				return ctx.Err()
-			case ch <- []*config.TargetGroup{{Source: source}}:
-				d.logger.Debugf("Removing group for %s", source)
+			case ch <- []*targetgroup.Group{{Source: source}}:
+				level.Debug(d.logger).Log("msg", "Removing group", "source", source)
 			}
 		}
 	}
@@ -178,7 +232,7 @@ func (d *Discovery) updateServices(ctx context.Context, ch chan<- []*config.Targ
 	return nil
 }
 
-func (d *Discovery) fetchTargetGroups() (map[string]*config.TargetGroup, error) {
+func (d *Discovery) fetchTargetGroups() (map[string]*targetgroup.Group, error) {
 	url := RandomAppsURL(d.servers)
 	apps, err := d.appsClient(d.client, url, d.token)
 	if err != nil {
@@ -209,7 +263,8 @@ type DockerContainer struct {
 
 // Container describes the runtime an app in running in.
 type Container struct {
-	Docker DockerContainer `json:"docker"`
+	Docker       DockerContainer `json:"docker"`
+	PortMappings []PortMappings  `json:"portMappings"`
 }
 
 // PortDefinitions describes which load balancer port you should access to access the service.
@@ -280,8 +335,8 @@ func RandomAppsURL(servers []string) string {
 }
 
 // AppsToTargetGroups takes an array of Marathon apps and converts them into target groups.
-func AppsToTargetGroups(apps *AppList) map[string]*config.TargetGroup {
-	tgroups := map[string]*config.TargetGroup{}
+func AppsToTargetGroups(apps *AppList) map[string]*targetgroup.Group {
+	tgroups := map[string]*targetgroup.Group{}
 	for _, a := range apps.Apps {
 		group := createTargetGroup(&a)
 		tgroups[group.Source] = group
@@ -289,13 +344,13 @@ func AppsToTargetGroups(apps *AppList) map[string]*config.TargetGroup {
 	return tgroups
 }
 
-func createTargetGroup(app *App) *config.TargetGroup {
+func createTargetGroup(app *App) *targetgroup.Group {
 	var (
 		targets = targetsForApp(app)
 		appName = model.LabelValue(app.ID)
 		image   = model.LabelValue(app.Container.Docker.Image)
 	)
-	tg := &config.TargetGroup{
+	tg := &targetgroup.Group{
 		Targets: targets,
 		Labels: model.LabelSet{
 			appLabel:   appName,
@@ -323,6 +378,7 @@ func targetsForApp(app *App) []model.LabelSet {
 			target := model.LabelSet{
 				model.AddressLabel: model.LabelValue(targetAddress),
 				taskLabel:          model.LabelValue(t.ID),
+				portIndexLabel:     model.LabelValue(strconv.Itoa(i)),
 			}
 			if i < len(app.PortDefinitions) {
 				for ln, lv := range app.PortDefinitions[i].Labels {
@@ -330,12 +386,23 @@ func targetsForApp(app *App) []model.LabelSet {
 					target[model.LabelName(ln)] = model.LabelValue(lv)
 				}
 			}
+			// Prior to Marathon 1.5 the port mappings could be found at the path
+			// "container.docker.portMappings".  When support for Marathon 1.4
+			// is dropped then this section of code can be removed.
 			if i < len(app.Container.Docker.PortMappings) {
 				for ln, lv := range app.Container.Docker.PortMappings[i].Labels {
 					ln = portMappingLabelPrefix + strutil.SanitizeLabelName(ln)
 					target[model.LabelName(ln)] = model.LabelValue(lv)
 				}
 			}
+			// In Marathon 1.5.x the container.docker.portMappings object was moved
+			// to container.portMappings.
+			if i < len(app.Container.PortMappings) {
+				for ln, lv := range app.Container.PortMappings[i].Labels {
+					ln = portMappingLabelPrefix + strutil.SanitizeLabelName(ln)
+					target[model.LabelName(ln)] = model.LabelValue(lv)
+				}
+			}
 			targets = append(targets, target)
 		}
 	}
