@@ -14,26 +14,32 @@
 package triton
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"io/ioutil"
 	"net/http"
 	"time"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
+	"github.com/mwitkow/go-conntrack"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/config"
+
+	config_util "github.com/prometheus/common/config"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
 	"github.com/prometheus/prometheus/util/httputil"
-	"golang.org/x/net/context"
+	yaml_util "github.com/prometheus/prometheus/util/yaml"
 )
 
 const (
 	tritonLabel             = model.MetaLabelPrefix + "triton_"
-	tritonLabelMachineId    = tritonLabel + "machine_id"
+	tritonLabelMachineID    = tritonLabel + "machine_id"
 	tritonLabelMachineAlias = tritonLabel + "machine_alias"
+	tritonLabelMachineBrand = tritonLabel + "machine_brand"
 	tritonLabelMachineImage = tritonLabel + "machine_image"
-	tritonLabelServerId     = tritonLabel + "server_id"
+	tritonLabelServerID     = tritonLabel + "server_id"
 	namespace               = "prometheus"
 )
 
@@ -48,39 +54,93 @@ var (
 			Name: "prometheus_sd_triton_refresh_duration_seconds",
 			Help: "The duration of a Triton-SD refresh in seconds.",
 		})
+	// DefaultSDConfig is the default Triton SD configuration.
+	DefaultSDConfig = SDConfig{
+		Port:            9163,
+		RefreshInterval: model.Duration(60 * time.Second),
+		Version:         1,
+	}
 )
 
+// SDConfig is the configuration for Triton based service discovery.
+type SDConfig struct {
+	Account         string                `yaml:"account"`
+	DNSSuffix       string                `yaml:"dns_suffix"`
+	Endpoint        string                `yaml:"endpoint"`
+	Port            int                   `yaml:"port"`
+	RefreshInterval model.Duration        `yaml:"refresh_interval,omitempty"`
+	TLSConfig       config_util.TLSConfig `yaml:"tls_config,omitempty"`
+	Version         int                   `yaml:"version"`
+	// Catches all undefined fields and must be empty after parsing.
+	XXX map[string]interface{} `yaml:",inline"`
+}
+
+// UnmarshalYAML implements the yaml.Unmarshaler interface.
+func (c *SDConfig) UnmarshalYAML(unmarshal func(interface{}) error) error {
+	*c = DefaultSDConfig
+	type plain SDConfig
+	err := unmarshal((*plain)(c))
+	if err != nil {
+		return err
+	}
+	if c.Account == "" {
+		return fmt.Errorf("Triton SD configuration requires an account")
+	}
+	if c.DNSSuffix == "" {
+		return fmt.Errorf("Triton SD configuration requires a dns_suffix")
+	}
+	if c.Endpoint == "" {
+		return fmt.Errorf("Triton SD configuration requires an endpoint")
+	}
+	if c.RefreshInterval <= 0 {
+		return fmt.Errorf("Triton SD configuration requires RefreshInterval to be a positive integer")
+	}
+	return yaml_util.CheckOverflow(c.XXX, "triton_sd_config")
+}
+
 func init() {
 	prometheus.MustRegister(refreshFailuresCount)
 	prometheus.MustRegister(refreshDuration)
 }
 
+// DiscoveryResponse models a JSON response from the Triton discovery.
 type DiscoveryResponse struct {
 	Containers []struct {
 		ServerUUID  string `json:"server_uuid"`
 		VMAlias     string `json:"vm_alias"`
+		VMBrand     string `json:"vm_brand"`
 		VMImageUUID string `json:"vm_image_uuid"`
 		VMUUID      string `json:"vm_uuid"`
 	} `json:"containers"`
 }
 
 // Discovery periodically performs Triton-SD requests. It implements
-// the TargetProvider interface.
+// the Discoverer interface.
 type Discovery struct {
 	client   *http.Client
 	interval time.Duration
 	logger   log.Logger
-	sdConfig *config.TritonSDConfig
+	sdConfig *SDConfig
 }
 
 // New returns a new Discovery which periodically refreshes its targets.
-func New(logger log.Logger, conf *config.TritonSDConfig) (*Discovery, error) {
+func New(logger log.Logger, conf *SDConfig) (*Discovery, error) {
+	if logger == nil {
+		logger = log.NewNopLogger()
+	}
+
 	tls, err := httputil.NewTLSConfig(conf.TLSConfig)
 	if err != nil {
 		return nil, err
 	}
 
-	transport := &http.Transport{TLSClientConfig: tls}
+	transport := &http.Transport{
+		TLSClientConfig: tls,
+		DialContext: conntrack.NewDialContextFunc(
+			conntrack.DialWithTracing(),
+			conntrack.DialWithName("triton_sd"),
+		),
+	}
 	client := &http.Client{Transport: transport}
 
 	return &Discovery{
@@ -91,8 +151,8 @@ func New(logger log.Logger, conf *config.TritonSDConfig) (*Discovery, error) {
 	}, nil
 }
 
-// Run implements the TargetProvider interface.
-func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
+// Run implements the Discoverer interface.
+func (d *Discovery) Run(ctx context.Context, ch chan<- []*targetgroup.Group) {
 	defer close(ch)
 
 	ticker := time.NewTicker(d.interval)
@@ -101,9 +161,9 @@ func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 	// Get an initial set right away.
 	tg, err := d.refresh()
 	if err != nil {
-		d.logger.With("err", err).Error("Refreshing targets failed")
+		level.Error(d.logger).Log("msg", "Refreshing targets failed", "err", err)
 	} else {
-		ch <- []*config.TargetGroup{tg}
+		ch <- []*targetgroup.Group{tg}
 	}
 
 	for {
@@ -111,9 +171,9 @@ func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 		case <-ticker.C:
 			tg, err := d.refresh()
 			if err != nil {
-				d.logger.With("err", err).Error("Refreshing targets failed")
+				level.Error(d.logger).Log("msg", "Refreshing targets failed", "err", err)
 			} else {
-				ch <- []*config.TargetGroup{tg}
+				ch <- []*targetgroup.Group{tg}
 			}
 		case <-ctx.Done():
 			return
@@ -121,7 +181,7 @@ func (d *Discovery) Run(ctx context.Context, ch chan<- []*config.TargetGroup) {
 	}
 }
 
-func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
+func (d *Discovery) refresh() (tg *targetgroup.Group, err error) {
 	t0 := time.Now()
 	defer func() {
 		refreshDuration.Observe(time.Since(t0).Seconds())
@@ -131,7 +191,7 @@ func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
 	}()
 
 	var endpoint = fmt.Sprintf("https://%s:%d/v%d/discover", d.sdConfig.Endpoint, d.sdConfig.Port, d.sdConfig.Version)
-	tg = &config.TargetGroup{
+	tg = &targetgroup.Group{
 		Source: endpoint,
 	}
 
@@ -155,10 +215,11 @@ func (d *Discovery) refresh() (tg *config.TargetGroup, err error) {
 
 	for _, container := range dr.Containers {
 		labels := model.LabelSet{
-			tritonLabelMachineId:    model.LabelValue(container.VMUUID),
+			tritonLabelMachineID:    model.LabelValue(container.VMUUID),
 			tritonLabelMachineAlias: model.LabelValue(container.VMAlias),
+			tritonLabelMachineBrand: model.LabelValue(container.VMBrand),
 			tritonLabelMachineImage: model.LabelValue(container.VMImageUUID),
-			tritonLabelServerId:     model.LabelValue(container.ServerUUID),
+			tritonLabelServerID:     model.LabelValue(container.ServerUUID),
 		}
 		addr := fmt.Sprintf("%s.%s:%d", container.VMUUID, d.sdConfig.DNSSuffix, d.sdConfig.Port)
 		labels[model.AddressLabel] = model.LabelValue(addr)
