@@ -14,6 +14,7 @@
 package triton
 
 import (
+	"context"
 	"fmt"
 	"net"
 	"net/http"
@@ -23,15 +24,15 @@ import (
 	"testing"
 	"time"
 
-	"github.com/prometheus/common/log"
-	"github.com/prometheus/common/model"
-	"github.com/prometheus/prometheus/config"
 	"github.com/stretchr/testify/assert"
-	"golang.org/x/net/context"
+
+	"github.com/prometheus/common/config"
+	"github.com/prometheus/common/model"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
 )
 
 var (
-	conf = config.TritonSDConfig{
+	conf = SDConfig{
 		Account:         "testAccount",
 		DNSSuffix:       "triton.example.com",
 		Endpoint:        "127.0.0.1",
@@ -40,7 +41,7 @@ var (
 		RefreshInterval: 1,
 		TLSConfig:       config.TLSConfig{InsecureSkipVerify: true},
 	}
-	badconf = config.TritonSDConfig{
+	badconf = SDConfig{
 		Account:         "badTestAccount",
 		DNSSuffix:       "bad.triton.example.com",
 		Endpoint:        "127.0.0.1",
@@ -54,17 +55,14 @@ var (
 			CertFile:           "shouldnotexist.cert",
 		},
 	}
-	logger = log.Base()
 )
 
 func TestTritonSDNew(t *testing.T) {
-	td, err := New(logger, &conf)
+	td, err := New(nil, &conf)
 	assert.Nil(t, err)
 	assert.NotNil(t, td)
 	assert.NotNil(t, td.client)
 	assert.NotNil(t, td.interval)
-	assert.NotNil(t, td.logger)
-	assert.Equal(t, logger, td.logger, "td.logger equals logger")
 	assert.NotNil(t, td.sdConfig)
 	assert.Equal(t, conf.Account, td.sdConfig.Account)
 	assert.Equal(t, conf.DNSSuffix, td.sdConfig.DNSSuffix)
@@ -73,15 +71,15 @@ func TestTritonSDNew(t *testing.T) {
 }
 
 func TestTritonSDNewBadConfig(t *testing.T) {
-	td, err := New(logger, &badconf)
+	td, err := New(nil, &badconf)
 	assert.NotNil(t, err)
 	assert.Nil(t, td)
 }
 
 func TestTritonSDRun(t *testing.T) {
 	var (
-		td, err     = New(logger, &conf)
-		ch          = make(chan []*config.TargetGroup)
+		td, err     = New(nil, &conf)
+		ch          = make(chan []*targetgroup.Group)
 		ctx, cancel = context.WithCancel(context.Background())
 	)
 
@@ -132,7 +130,7 @@ func TestTritonSDRefreshMultipleTargets(t *testing.T) {
 
 func TestTritonSDRefreshNoServer(t *testing.T) {
 	var (
-		td, err = New(logger, &conf)
+		td, err = New(nil, &conf)
 	)
 	assert.Nil(t, err)
 	assert.NotNil(t, td)
@@ -146,7 +144,7 @@ func TestTritonSDRefreshNoServer(t *testing.T) {
 
 func testTritonSDRefresh(t *testing.T, dstr string) []model.LabelSet {
 	var (
-		td, err = New(logger, &conf)
+		td, err = New(nil, &conf)
 		s       = httptest.NewTLSServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 			fmt.Fprintln(w, dstr)
 		}))
