@@ -21,12 +21,15 @@ import (
 	"sort"
 	"time"
 
-	"github.com/prometheus/common/log"
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/common/model"
 )
 
 // Client allows sending batches of Prometheus samples to Graphite.
 type Client struct {
+	logger log.Logger
+
 	address   string
 	transport string
 	timeout   time.Duration
@@ -34,8 +37,12 @@ type Client struct {
 }
 
 // NewClient creates a new Client.
-func NewClient(address string, transport string, timeout time.Duration, prefix string) *Client {
+func NewClient(logger log.Logger, address string, transport string, timeout time.Duration, prefix string) *Client {
+	if logger == nil {
+		logger = log.NewNopLogger()
+	}
 	return &Client{
+		logger:    logger,
 		address:   address,
 		transport: transport,
 		timeout:   timeout,
@@ -86,8 +93,7 @@ func (c *Client) Write(samples model.Samples) error {
 		t := float64(s.Timestamp.UnixNano()) / 1e9
 		v := float64(s.Value)
 		if math.IsNaN(v) || math.IsInf(v, 0) {
-			log.Debugf("cannot send value %f to Graphite,"+
-				"skipping sample %#v", v, s)
+			level.Debug(c.logger).Log("msg", "cannot send value to Graphite, skipping sample", "value", v, "sample", s)
 			continue
 		}
 		fmt.Fprintf(&buf, "%s %f %f\n", k, v, t)
