@@ -15,15 +15,18 @@ package promql
 
 import (
 	"fmt"
+	"math"
+	"os"
 	"runtime"
+	"sort"
 	"strconv"
 	"strings"
 	"time"
 
-	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
+	"github.com/prometheus/prometheus/pkg/labels"
+	"github.com/prometheus/prometheus/pkg/value"
 
-	"github.com/prometheus/prometheus/storage/metric"
 	"github.com/prometheus/prometheus/util/strutil"
 )
 
@@ -73,7 +76,7 @@ func ParseExpr(input string) (Expr, error) {
 }
 
 // ParseMetric parses the input into a metric
-func ParseMetric(input string) (m model.Metric, err error) {
+func ParseMetric(input string) (m labels.Labels, err error) {
 	p := newParser(input)
 	defer p.recover(&err)
 
@@ -86,7 +89,7 @@ func ParseMetric(input string) (m model.Metric, err error) {
 
 // ParseMetricSelector parses the provided textual metric selector into a list of
 // label matchers.
-func ParseMetricSelector(input string) (m metric.LabelMatchers, err error) {
+func ParseMetricSelector(input string) (m []*labels.Matcher, err error) {
 	p := newParser(input)
 	defer p.recover(&err)
 
@@ -94,7 +97,7 @@ func ParseMetricSelector(input string) (m metric.LabelMatchers, err error) {
 	if t := p.peek().typ; t == itemMetricIdentifier || t == itemIdentifier {
 		name = p.next().val
 	}
-	vs := p.vectorSelector(name)
+	vs := p.VectorSelector(name)
 	if p.peek().typ != itemEOF {
 		p.errorf("could not parse remaining input %.15q...", p.lex.input[p.lex.lastPos:])
 	}
@@ -102,7 +105,7 @@ func ParseMetricSelector(input string) (m metric.LabelMatchers, err error) {
 }
 
 // parseSeriesDesc parses the description of a time series.
-func parseSeriesDesc(input string) (model.Metric, []sequenceValue, error) {
+func parseSeriesDesc(input string) (labels.Labels, []sequenceValue, error) {
 	p := newParser(input)
 	p.lex.seriesDesc = true
 
@@ -153,7 +156,7 @@ func (p *parser) parseExpr() (expr Expr, err error) {
 
 // sequenceValue is an omittable value in a sequence of time series values.
 type sequenceValue struct {
-	value   model.SampleValue
+	value   float64
 	omitted bool
 }
 
@@ -161,11 +164,11 @@ func (v sequenceValue) String() string {
 	if v.omitted {
 		return "_"
 	}
-	return v.value.String()
+	return fmt.Sprintf("%f", v.value)
 }
 
 // parseSeriesDesc parses a description of a time series into its metric and value sequence.
-func (p *parser) parseSeriesDesc() (m model.Metric, vals []sequenceValue, err error) {
+func (p *parser) parseSeriesDesc() (m labels.Labels, vals []sequenceValue, err error) {
 	defer p.recover(&err)
 
 	m = p.metric()
@@ -200,17 +203,25 @@ func (p *parser) parseSeriesDesc() (m model.Metric, vals []sequenceValue, err er
 				sign = -1
 			}
 		}
-		k := sign * p.number(p.expect(itemNumber, ctx).val)
+		var k float64
+		if t := p.peek().typ; t == itemNumber {
+			k = sign * p.number(p.expect(itemNumber, ctx).val)
+		} else if t == itemIdentifier && p.peek().val == "stale" {
+			p.next()
+			k = math.Float64frombits(value.StaleNaN)
+		} else {
+			p.errorf("expected number or 'stale' in %s but got %s", ctx, t.desc())
+		}
 		vals = append(vals, sequenceValue{
-			value: model.SampleValue(k),
+			value: k,
 		})
 
 		// If there are no offset repetitions specified, proceed with the next value.
-		if t := p.peek().typ; t == itemNumber || t == itemBlank {
+		if t := p.peek(); t.typ == itemNumber || t.typ == itemBlank || t.typ == itemIdentifier && t.val == "stale" {
 			continue
-		} else if t == itemEOF {
+		} else if t.typ == itemEOF {
 			break
-		} else if t != itemADD && t != itemSUB {
+		} else if t.typ != itemADD && t.typ != itemSUB {
 			p.errorf("expected next value or relative expansion in %s but got %s", ctx, t.desc())
 		}
 
@@ -230,7 +241,7 @@ func (p *parser) parseSeriesDesc() (m model.Metric, vals []sequenceValue, err er
 		for i := uint64(0); i < times; i++ {
 			k += offset
 			vals = append(vals, sequenceValue{
-				value: model.SampleValue(k),
+				value: k,
 			})
 		}
 	}
@@ -331,7 +342,7 @@ func (p *parser) recover(errp *error) {
 			buf := make([]byte, 64<<10)
 			buf = buf[:runtime.Stack(buf, false)]
 
-			log.Errorf("parser panic: %v\n%s", e, buf)
+			fmt.Fprintf(os.Stderr, "parser panic: %v\n%s", e, buf)
 			*errp = errUnexpected
 		} else {
 			*errp = e.(error)
@@ -366,7 +377,7 @@ func (p *parser) alertStmt() *AlertStmt {
 
 	p.expect(itemAlert, ctx)
 	name := p.expect(itemIdentifier, ctx)
-	// Alerts require a vector typed expression.
+	// Alerts require a Vector typed expression.
 	p.expect(itemIf, ctx)
 	expr := p.expr()
 
@@ -385,12 +396,12 @@ func (p *parser) alertStmt() *AlertStmt {
 	}
 
 	var (
-		labels      = model.LabelSet{}
-		annotations = model.LabelSet{}
+		lset        labels.Labels
+		annotations labels.Labels
 	)
 	if p.peek().typ == itemLabels {
 		p.expect(itemLabels, ctx)
-		labels = p.labelSet()
+		lset = p.labelSet()
 	}
 	if p.peek().typ == itemAnnotations {
 		p.expect(itemAnnotations, ctx)
@@ -401,7 +412,7 @@ func (p *parser) alertStmt() *AlertStmt {
 		Name:        name.val,
 		Expr:        expr,
 		Duration:    duration,
-		Labels:      labels,
+		Labels:      lset,
 		Annotations: annotations,
 	}
 }
@@ -412,7 +423,7 @@ func (p *parser) recordStmt() *RecordStmt {
 
 	name := p.expectOneOf(itemIdentifier, itemMetricIdentifier, ctx).val
 
-	var lset model.LabelSet
+	var lset labels.Labels
 	if p.peek().typ == itemLeftBrace {
 		lset = p.labelSet()
 	}
@@ -504,7 +515,7 @@ func (p *parser) balance(lhs Expr, op itemType, rhs Expr, vecMatching *VectorMat
 		precd := lhsBE.Op.precedence() - op.precedence()
 		if (precd < 0) || (precd == 0 && op.isRightAssociative()) {
 			balanced := p.balance(lhsBE.RHS, op, rhs, vecMatching, returnBool)
-			if lhsBE.Op.isComparisonOperator() && !lhsBE.ReturnBool && balanced.Type() == model.ValScalar && lhsBE.LHS.Type() == model.ValScalar {
+			if lhsBE.Op.isComparisonOperator() && !lhsBE.ReturnBool && balanced.Type() == ValueTypeScalar && lhsBE.LHS.Type() == ValueTypeScalar {
 				p.errorf("comparisons between scalars must use BOOL modifier")
 			}
 			return &BinaryExpr{
@@ -516,7 +527,7 @@ func (p *parser) balance(lhs Expr, op itemType, rhs Expr, vecMatching *VectorMat
 			}
 		}
 	}
-	if op.isComparisonOperator() && !returnBool && rhs.Type() == model.ValScalar && lhs.Type() == model.ValScalar {
+	if op.isComparisonOperator() && !returnBool && rhs.Type() == ValueTypeScalar && lhs.Type() == ValueTypeScalar {
 		p.errorf("comparisons between scalars must use BOOL modifier")
 	}
 	return &BinaryExpr{
@@ -530,7 +541,7 @@ func (p *parser) balance(lhs Expr, op itemType, rhs Expr, vecMatching *VectorMat
 
 // unaryExpr parses a unary expression.
 //
-//		<vector_selector> | <matrix_selector> | (+|-) <number_literal> | '(' <expr> ')'
+//		<Vector_selector> | <Matrix_selector> | (+|-) <number_literal> | '(' <expr> ')'
 //
 func (p *parser) unaryExpr() Expr {
 	switch t := p.peek(); t.typ {
@@ -582,10 +593,10 @@ func (p *parser) unaryExpr() Expr {
 	return e
 }
 
-// rangeSelector parses a matrix (a.k.a. range) selector based on a given
-// vector selector.
+// rangeSelector parses a Matrix (a.k.a. range) selector based on a given
+// Vector selector.
 //
-//		<vector_selector> '[' <duration> ']'
+//		<Vector_selector> '[' <duration> ']'
 //
 func (p *parser) rangeSelector(vs *VectorSelector) *MatrixSelector {
 	const ctx = "range selector"
@@ -625,13 +636,13 @@ func (p *parser) number(val string) float64 {
 
 // primaryExpr parses a primary expression.
 //
-//		<metric_name> | <function_call> | <vector_aggregation> | <literal>
+//		<metric_name> | <function_call> | <Vector_aggregation> | <literal>
 //
 func (p *parser) primaryExpr() Expr {
 	switch t := p.next(); {
 	case t.typ == itemNumber:
 		f := p.number(t.val)
-		return &NumberLiteral{model.SampleValue(f)}
+		return &NumberLiteral{f}
 
 	case t.typ == itemString:
 		return &StringLiteral{p.unquoteString(t.val)}
@@ -639,7 +650,7 @@ func (p *parser) primaryExpr() Expr {
 	case t.typ == itemLeftBrace:
 		// Metric selector without metric name.
 		p.backup()
-		return p.vectorSelector("")
+		return p.VectorSelector("")
 
 	case t.typ == itemIdentifier:
 		// Check for function call.
@@ -649,7 +660,7 @@ func (p *parser) primaryExpr() Expr {
 		fallthrough // Else metric selector.
 
 	case t.typ == itemMetricIdentifier:
-		return p.vectorSelector(t.val)
+		return p.VectorSelector(t.val)
 
 	case t.typ.isAggregator():
 		p.backup()
@@ -665,19 +676,19 @@ func (p *parser) primaryExpr() Expr {
 //
 //		'(' <label_name>, ... ')'
 //
-func (p *parser) labels() model.LabelNames {
+func (p *parser) labels() []string {
 	const ctx = "grouping opts"
 
 	p.expect(itemLeftParen, ctx)
 
-	labels := model.LabelNames{}
+	labels := []string{}
 	if p.peek().typ != itemRightParen {
 		for {
 			id := p.next()
 			if !isLabel(id.val) {
 				p.errorf("unexpected %s in %s, expected label", id.desc(), ctx)
 			}
-			labels = append(labels, model.LabelName(id.val))
+			labels = append(labels, id.val)
 
 			if p.peek().typ != itemComma {
 				break
@@ -692,8 +703,8 @@ func (p *parser) labels() model.LabelNames {
 
 // aggrExpr parses an aggregation expression.
 //
-//		<aggr_op> (<vector_expr>) [by <labels>] [keep_common]
-//		<aggr_op> [by <labels>] [keep_common] (<vector_expr>)
+//		<aggr_op> (<Vector_expr>) [by|without <labels>]
+//		<aggr_op> [by|without <labels>] (<Vector_expr>)
 //
 func (p *parser) aggrExpr() *AggregateExpr {
 	const ctx = "aggregation"
@@ -702,8 +713,8 @@ func (p *parser) aggrExpr() *AggregateExpr {
 	if !agop.typ.isAggregator() {
 		p.errorf("expected aggregation operator but got %s", agop)
 	}
-	var grouping model.LabelNames
-	var keepCommon, without bool
+	var grouping []string
+	var without bool
 
 	modifiersFirst := false
 
@@ -715,11 +726,6 @@ func (p *parser) aggrExpr() *AggregateExpr {
 		grouping = p.labels()
 		modifiersFirst = true
 	}
-	if p.peek().typ == itemKeepCommon {
-		p.next()
-		keepCommon = true
-		modifiersFirst = true
-	}
 
 	p.expect(itemLeftParen, ctx)
 	var param Expr
@@ -741,23 +747,14 @@ func (p *parser) aggrExpr() *AggregateExpr {
 			p.next()
 			grouping = p.labels()
 		}
-		if p.peek().typ == itemKeepCommon {
-			p.next()
-			keepCommon = true
-		}
-	}
-
-	if keepCommon && without {
-		p.errorf("cannot use 'keep_common' with 'without'")
 	}
 
 	return &AggregateExpr{
-		Op:               agop.typ,
-		Expr:             e,
-		Param:            param,
-		Grouping:         grouping,
-		Without:          without,
-		KeepCommonLabels: keepCommon,
+		Op:       agop.typ,
+		Expr:     e,
+		Param:    param,
+		Grouping: grouping,
+		Without:  without,
 	}
 }
 
@@ -802,22 +799,22 @@ func (p *parser) call(name string) *Call {
 //
 //		'{' [ <labelname> '=' <match_string>, ... ] '}'
 //
-func (p *parser) labelSet() model.LabelSet {
-	set := model.LabelSet{}
+func (p *parser) labelSet() labels.Labels {
+	set := []labels.Label{}
 	for _, lm := range p.labelMatchers(itemEQL) {
-		set[lm.Name] = lm.Value
+		set = append(set, labels.Label{Name: lm.Name, Value: lm.Value})
 	}
-	return set
+	return labels.New(set...)
 }
 
 // labelMatchers parses a set of label matchers.
 //
 //		'{' [ <labelname> <match_op> <match_string>, ... ] '}'
 //
-func (p *parser) labelMatchers(operators ...itemType) metric.LabelMatchers {
+func (p *parser) labelMatchers(operators ...itemType) []*labels.Matcher {
 	const ctx = "label matching"
 
-	matchers := metric.LabelMatchers{}
+	matchers := []*labels.Matcher{}
 
 	p.expect(itemLeftBrace, ctx)
 
@@ -847,25 +844,21 @@ func (p *parser) labelMatchers(operators ...itemType) metric.LabelMatchers {
 		val := p.unquoteString(p.expect(itemString, ctx).val)
 
 		// Map the item to the respective match type.
-		var matchType metric.MatchType
+		var matchType labels.MatchType
 		switch op {
 		case itemEQL:
-			matchType = metric.Equal
+			matchType = labels.MatchEqual
 		case itemNEQ:
-			matchType = metric.NotEqual
+			matchType = labels.MatchNotEqual
 		case itemEQLRegex:
-			matchType = metric.RegexMatch
+			matchType = labels.MatchRegexp
 		case itemNEQRegex:
-			matchType = metric.RegexNoMatch
+			matchType = labels.MatchNotRegexp
 		default:
 			p.errorf("item %q is not a metric match type", op)
 		}
 
-		m, err := metric.NewLabelMatcher(
-			matchType,
-			model.LabelName(label.val),
-			model.LabelValue(val),
-		)
+		m, err := labels.NewMatcher(matchType, label.val, val)
 		if err != nil {
 			p.error(err)
 		}
@@ -898,9 +891,9 @@ func (p *parser) labelMatchers(operators ...itemType) metric.LabelMatchers {
 //		<label_set>
 //		<metric_identifier> [<label_set>]
 //
-func (p *parser) metric() model.Metric {
+func (p *parser) metric() labels.Labels {
 	name := ""
-	m := model.Metric{}
+	var m labels.Labels
 
 	t := p.peek().typ
 	if t == itemIdentifier || t == itemMetricIdentifier {
@@ -911,10 +904,11 @@ func (p *parser) metric() model.Metric {
 		p.errorf("missing metric name or metric selector")
 	}
 	if t == itemLeftBrace {
-		m = model.Metric(p.labelSet())
+		m = p.labelSet()
 	}
 	if name != "" {
-		m[model.MetricNameLabel] = model.LabelValue(name)
+		m = append(m, labels.Label{Name: labels.MetricName, Value: name})
+		sort.Sort(m)
 	}
 	return m
 }
@@ -937,13 +931,13 @@ func (p *parser) offset() time.Duration {
 	return offset
 }
 
-// vectorSelector parses a new (instant) vector selector.
+// VectorSelector parses a new (instant) vector selector.
 //
 //		<metric_identifier> [<label_matchers>]
 //		[<metric_identifier>] <label_matchers>
 //
-func (p *parser) vectorSelector(name string) *VectorSelector {
-	var matchers metric.LabelMatchers
+func (p *parser) VectorSelector(name string) *VectorSelector {
+	var matchers []*labels.Matcher
 	// Parse label matching if any.
 	if t := p.peek(); t.typ == itemLeftBrace {
 		matchers = p.labelMatchers(itemEQL, itemNEQ, itemEQLRegex, itemNEQRegex)
@@ -951,12 +945,12 @@ func (p *parser) vectorSelector(name string) *VectorSelector {
 	// Metric name must not be set in the label matchers and before at the same time.
 	if name != "" {
 		for _, m := range matchers {
-			if m.Name == model.MetricNameLabel {
+			if m.Name == labels.MetricName {
 				p.errorf("metric name must not be set twice: %q or %q", name, m.Value)
 			}
 		}
 		// Set name label matching.
-		m, err := metric.NewLabelMatcher(metric.Equal, model.MetricNameLabel, model.LabelValue(name))
+		m, err := labels.NewMatcher(labels.MatchEqual, labels.MetricName, name)
 		if err != nil {
 			panic(err) // Must not happen with metric.Equal.
 		}
@@ -966,11 +960,11 @@ func (p *parser) vectorSelector(name string) *VectorSelector {
 	if len(matchers) == 0 {
 		p.errorf("vector selector must contain label matchers or metric name")
 	}
-	// A vector selector must contain at least one non-empty matcher to prevent
+	// A Vector selector must contain at least one non-empty matcher to prevent
 	// implicit selection of all metrics (e.g. by a typo).
 	notEmpty := false
 	for _, lm := range matchers {
-		if !lm.MatchesEmptyString() {
+		if !lm.Matches("") {
 			notEmpty = true
 			break
 		}
@@ -987,7 +981,7 @@ func (p *parser) vectorSelector(name string) *VectorSelector {
 
 // expectType checks the type of the node and raises an error if it
 // is not of the expected type.
-func (p *parser) expectType(node Node, want model.ValueType, context string) {
+func (p *parser) expectType(node Node, want ValueType, context string) {
 	t := p.checkType(node)
 	if t != want {
 		p.errorf("expected type %s in %s, got %s", documentedType(want), context, documentedType(t))
@@ -999,12 +993,12 @@ func (p *parser) expectType(node Node, want model.ValueType, context string) {
 //
 // Some of these checks are redundant as the the parsing stage does not allow
 // them, but the costs are small and might reveal errors when making changes.
-func (p *parser) checkType(node Node) (typ model.ValueType) {
+func (p *parser) checkType(node Node) (typ ValueType) {
 	// For expressions the type is determined by their Type function.
 	// Statements and lists do not have a type but are not invalid either.
 	switch n := node.(type) {
 	case Statements, Expressions, Statement:
-		typ = model.ValNone
+		typ = ValueTypeNone
 	case Expr:
 		typ = n.Type()
 	default:
@@ -1016,27 +1010,27 @@ func (p *parser) checkType(node Node) (typ model.ValueType) {
 	switch n := node.(type) {
 	case Statements:
 		for _, s := range n {
-			p.expectType(s, model.ValNone, "statement list")
+			p.expectType(s, ValueTypeNone, "statement list")
 		}
 	case *AlertStmt:
-		p.expectType(n.Expr, model.ValVector, "alert statement")
+		p.expectType(n.Expr, ValueTypeVector, "alert statement")
 
 	case *EvalStmt:
 		ty := p.checkType(n.Expr)
-		if ty == model.ValNone {
+		if ty == ValueTypeNone {
 			p.errorf("evaluation statement must have a valid expression type but got %s", documentedType(ty))
 		}
 
 	case *RecordStmt:
 		ty := p.checkType(n.Expr)
-		if ty != model.ValVector && ty != model.ValScalar {
+		if ty != ValueTypeVector && ty != ValueTypeScalar {
 			p.errorf("record statement must have a valid expression of type instant vector or scalar but got %s", documentedType(ty))
 		}
 
 	case Expressions:
 		for _, e := range n {
 			ty := p.checkType(e)
-			if ty == model.ValNone {
+			if ty == ValueTypeNone {
 				p.errorf("expression must have a valid expression type but got %s", documentedType(ty))
 			}
 		}
@@ -1044,12 +1038,12 @@ func (p *parser) checkType(node Node) (typ model.ValueType) {
 		if !n.Op.isAggregator() {
 			p.errorf("aggregation operator expected in aggregation expression but got %q", n.Op)
 		}
-		p.expectType(n.Expr, model.ValVector, "aggregation expression")
+		p.expectType(n.Expr, ValueTypeVector, "aggregation expression")
 		if n.Op == itemTopK || n.Op == itemBottomK || n.Op == itemQuantile {
-			p.expectType(n.Param, model.ValScalar, "aggregation parameter")
+			p.expectType(n.Param, ValueTypeScalar, "aggregation parameter")
 		}
 		if n.Op == itemCountValues {
-			p.expectType(n.Param, model.ValString, "aggregation parameter")
+			p.expectType(n.Param, ValueTypeString, "aggregation parameter")
 		}
 
 	case *BinaryExpr:
@@ -1059,17 +1053,17 @@ func (p *parser) checkType(node Node) (typ model.ValueType) {
 		if !n.Op.isOperator() {
 			p.errorf("binary expression does not support operator %q", n.Op)
 		}
-		if (lt != model.ValScalar && lt != model.ValVector) || (rt != model.ValScalar && rt != model.ValVector) {
+		if (lt != ValueTypeScalar && lt != ValueTypeVector) || (rt != ValueTypeScalar && rt != ValueTypeVector) {
 			p.errorf("binary expression must contain only scalar and instant vector types")
 		}
 
-		if (lt != model.ValVector || rt != model.ValVector) && n.VectorMatching != nil {
+		if (lt != ValueTypeVector || rt != ValueTypeVector) && n.VectorMatching != nil {
 			if len(n.VectorMatching.MatchingLabels) > 0 {
 				p.errorf("vector matching only allowed between instant vectors")
 			}
 			n.VectorMatching = nil
 		} else {
-			// Both operands are vectors.
+			// Both operands are Vectors.
 			if n.Op.isSetOperator() {
 				if n.VectorMatching.Card == CardOneToMany || n.VectorMatching.Card == CardManyToOne {
 					p.errorf("no grouping allowed for %q operation", n.Op)
@@ -1080,7 +1074,7 @@ func (p *parser) checkType(node Node) (typ model.ValueType) {
 			}
 		}
 
-		if (lt == model.ValScalar || rt == model.ValScalar) && n.Op.isSetOperator() {
+		if (lt == ValueTypeScalar || rt == ValueTypeScalar) && n.Op.isSetOperator() {
 			p.errorf("set operator %q not allowed in binary scalar expression", n.Op)
 		}
 
@@ -1113,7 +1107,7 @@ func (p *parser) checkType(node Node) (typ model.ValueType) {
 		if n.Op != itemADD && n.Op != itemSUB {
 			p.errorf("only + and - operators allowed for unary expressions")
 		}
-		if t := p.checkType(n.Expr); t != model.ValScalar && t != model.ValVector {
+		if t := p.checkType(n.Expr); t != ValueTypeScalar && t != ValueTypeVector {
 			p.errorf("unary expression only allowed on expressions of type scalar or instant vector, got %q", documentedType(t))
 		}
 
