@@ -20,8 +20,7 @@ import (
 	"time"
 
 	"github.com/prometheus/common/model"
-
-	"github.com/prometheus/prometheus/storage/metric"
+	"github.com/prometheus/prometheus/pkg/labels"
 )
 
 // Tree returns a string of the tree structure of the given node.
@@ -147,10 +146,7 @@ func (node *AggregateExpr) String() string {
 		} else {
 			format = "%s BY (%s)"
 		}
-		aggrString = fmt.Sprintf(format, aggrString, node.Grouping)
-	}
-	if node.KeepCommonLabels {
-		aggrString += " KEEP_COMMON"
+		aggrString = fmt.Sprintf(format, aggrString, strings.Join(node.Grouping, ", "))
 	}
 	return aggrString
 }
@@ -165,9 +161,9 @@ func (node *BinaryExpr) String() string {
 	vm := node.VectorMatching
 	if vm != nil && (len(vm.MatchingLabels) > 0 || vm.On) {
 		if vm.On {
-			matching = fmt.Sprintf(" ON(%s)", vm.MatchingLabels)
+			matching = fmt.Sprintf(" ON(%s)", strings.Join(vm.MatchingLabels, ", "))
 		} else {
-			matching = fmt.Sprintf(" IGNORING(%s)", vm.MatchingLabels)
+			matching = fmt.Sprintf(" IGNORING(%s)", strings.Join(vm.MatchingLabels, ", "))
 		}
 		if vm.Card == CardManyToOne || vm.Card == CardOneToMany {
 			matching += " GROUP_"
@@ -176,7 +172,7 @@ func (node *BinaryExpr) String() string {
 			} else {
 				matching += "RIGHT"
 			}
-			matching += fmt.Sprintf("(%s)", vm.Include)
+			matching += fmt.Sprintf("(%s)", strings.Join(vm.Include, ", "))
 		}
 	}
 	return fmt.Sprintf("%s %s%s%s %s", node.LHS, node.Op, returnBool, matching, node.RHS)
@@ -218,7 +214,7 @@ func (node *VectorSelector) String() string {
 	labelStrings := make([]string, 0, len(node.LabelMatchers)-1)
 	for _, matcher := range node.LabelMatchers {
 		// Only include the __name__ label if its no equality matching.
-		if matcher.Name == model.MetricNameLabel && matcher.Type == metric.Equal {
+		if matcher.Name == labels.MetricName && matcher.Type == labels.MatchEqual {
 			continue
 		}
 		labelStrings = append(labelStrings, matcher.String())
