@@ -14,6 +14,7 @@
 package retrieval
 
 import (
+	"errors"
 	"fmt"
 	"hash/fnv"
 	"net"
@@ -25,7 +26,10 @@ import (
 	"github.com/prometheus/common/model"
 
 	"github.com/prometheus/prometheus/config"
-	"github.com/prometheus/prometheus/relabel"
+	"github.com/prometheus/prometheus/discovery/targetgroup"
+	"github.com/prometheus/prometheus/pkg/labels"
+	"github.com/prometheus/prometheus/pkg/relabel"
+	"github.com/prometheus/prometheus/pkg/value"
 	"github.com/prometheus/prometheus/storage"
 )
 
@@ -42,9 +46,9 @@ const (
 // Target refers to a singular HTTP or HTTPS endpoint.
 type Target struct {
 	// Labels before any processing.
-	discoveredLabels model.LabelSet
+	discoveredLabels labels.Labels
 	// Any labels that are added to this target and its metrics.
-	labels model.LabelSet
+	labels labels.Labels
 	// Additional URL parmeters that are part of the target URL.
 	params url.Values
 
@@ -55,7 +59,7 @@ type Target struct {
 }
 
 // NewTarget creates a reasonably configured target for querying.
-func NewTarget(labels, discoveredLabels model.LabelSet, params url.Values) *Target {
+func NewTarget(labels, discoveredLabels labels.Labels, params url.Values) *Target {
 	return &Target{
 		labels:           labels,
 		discoveredLabels: discoveredLabels,
@@ -71,7 +75,7 @@ func (t *Target) String() string {
 // hash returns an identifying hash for the target.
 func (t *Target) hash() uint64 {
 	h := fnv.New64a()
-	h.Write([]byte(t.labels.Fingerprint().String()))
+	h.Write([]byte(fmt.Sprintf("%016d", t.labels.Hash())))
 	h.Write([]byte(t.URL().String()))
 
 	return h.Sum64()
@@ -94,19 +98,21 @@ func (t *Target) offset(interval time.Duration) time.Duration {
 }
 
 // Labels returns a copy of the set of all public labels of the target.
-func (t *Target) Labels() model.LabelSet {
-	lset := make(model.LabelSet, len(t.labels))
-	for ln, lv := range t.labels {
-		if !strings.HasPrefix(string(ln), model.ReservedLabelPrefix) {
-			lset[ln] = lv
+func (t *Target) Labels() labels.Labels {
+	lset := make(labels.Labels, 0, len(t.labels))
+	for _, l := range t.labels {
+		if !strings.HasPrefix(l.Name, model.ReservedLabelPrefix) {
+			lset = append(lset, l)
 		}
 	}
 	return lset
 }
 
 // DiscoveredLabels returns a copy of the target's labels before any processing.
-func (t *Target) DiscoveredLabels() model.LabelSet {
-	return t.discoveredLabels.Clone()
+func (t *Target) DiscoveredLabels() labels.Labels {
+	lset := make(labels.Labels, len(t.discoveredLabels))
+	copy(lset, t.discoveredLabels)
+	return lset
 }
 
 // URL returns a copy of the target's URL.
@@ -117,23 +123,23 @@ func (t *Target) URL() *url.URL {
 		params[k] = make([]string, len(v))
 		copy(params[k], v)
 	}
-	for k, v := range t.labels {
-		if !strings.HasPrefix(string(k), model.ParamLabelPrefix) {
+	for _, l := range t.labels {
+		if !strings.HasPrefix(l.Name, model.ParamLabelPrefix) {
 			continue
 		}
-		ks := string(k[len(model.ParamLabelPrefix):])
+		ks := l.Name[len(model.ParamLabelPrefix):]
 
 		if len(params[ks]) > 0 {
-			params[ks][0] = string(v)
+			params[ks][0] = string(l.Value)
 		} else {
-			params[ks] = []string{string(v)}
+			params[ks] = []string{l.Value}
 		}
 	}
 
 	return &url.URL{
-		Scheme:   string(t.labels[model.SchemeLabel]),
-		Host:     string(t.labels[model.AddressLabel]),
-		Path:     string(t.labels[model.MetricsPathLabel]),
+		Scheme:   string(t.labels.Get(model.SchemeLabel)),
+		Host:     string(t.labels.Get(model.AddressLabel)),
+		Path:     string(t.labels.Get(model.MetricsPathLabel)),
 		RawQuery: params.Encode(),
 	}
 }
@@ -183,119 +189,104 @@ func (ts Targets) Len() int           { return len(ts) }
 func (ts Targets) Less(i, j int) bool { return ts[i].URL().String() < ts[j].URL().String() }
 func (ts Targets) Swap(i, j int)      { ts[i], ts[j] = ts[j], ts[i] }
 
-// Merges the ingested sample's metric with the label set. On a collision the
-// value of the ingested label is stored in a label prefixed with 'exported_'.
-type ruleLabelsAppender struct {
-	storage.SampleAppender
-	labels model.LabelSet
+var errSampleLimit = errors.New("sample limit exceeded")
+
+// limitAppender limits the number of total appended samples in a batch.
+type limitAppender struct {
+	storage.Appender
+
+	limit int
+	i     int
 }
 
-func (app ruleLabelsAppender) Append(s *model.Sample) error {
-	for ln, lv := range app.labels {
-		if v, ok := s.Metric[ln]; ok && v != "" {
-			s.Metric[model.ExportedLabelPrefix+ln] = v
-		}
-		s.Metric[ln] = lv
-	}
-
-	return app.SampleAppender.Append(s)
-}
-
-type honorLabelsAppender struct {
-	storage.SampleAppender
-	labels model.LabelSet
-}
-
-// Merges the sample's metric with the given labels if the label is not
-// already present in the metric.
-// This also considers labels explicitly set to the empty string.
-func (app honorLabelsAppender) Append(s *model.Sample) error {
-	for ln, lv := range app.labels {
-		if _, ok := s.Metric[ln]; !ok {
-			s.Metric[ln] = lv
+func (app *limitAppender) Add(lset labels.Labels, t int64, v float64) (uint64, error) {
+	if !value.IsStaleNaN(v) {
+		app.i++
+		if app.i > app.limit {
+			return 0, errSampleLimit
 		}
 	}
-
-	return app.SampleAppender.Append(s)
-}
-
-// Applies a set of relabel configurations to the sample's metric
-// before actually appending it.
-type relabelAppender struct {
-	storage.SampleAppender
-	relabelings []*config.RelabelConfig
-}
-
-func (app relabelAppender) Append(s *model.Sample) error {
-	labels := relabel.Process(model.LabelSet(s.Metric), app.relabelings...)
-
-	// Check if the timeseries was dropped.
-	if labels == nil {
-		return nil
+	ref, err := app.Appender.Add(lset, t, v)
+	if err != nil {
+		return 0, err
 	}
-	s.Metric = model.Metric(labels)
-
-	return app.SampleAppender.Append(s)
+	return ref, nil
 }
 
-// bufferAppender appends samples to the given buffer.
-type bufferAppender struct {
-	buffer model.Samples
+func (app *limitAppender) AddFast(lset labels.Labels, ref uint64, t int64, v float64) error {
+	if !value.IsStaleNaN(v) {
+		app.i++
+		if app.i > app.limit {
+			return errSampleLimit
+		}
+	}
+	err := app.Appender.AddFast(lset, ref, t, v)
+	return err
 }
 
-func (app *bufferAppender) Append(s *model.Sample) error {
-	app.buffer = append(app.buffer, s)
-	return nil
+type timeLimitAppender struct {
+	storage.Appender
+
+	maxTime int64
 }
 
-func (app *bufferAppender) NeedsThrottling() bool { return false }
+func (app *timeLimitAppender) Add(lset labels.Labels, t int64, v float64) (uint64, error) {
+	if t > app.maxTime {
+		return 0, storage.ErrOutOfBounds
+	}
 
-// countingAppender counts the samples appended to the underlying appender.
-type countingAppender struct {
-	storage.SampleAppender
-	count int
+	ref, err := app.Appender.Add(lset, t, v)
+	if err != nil {
+		return 0, err
+	}
+	return ref, nil
 }
 
-func (app *countingAppender) Append(s *model.Sample) error {
-	app.count++
-	return app.SampleAppender.Append(s)
+func (app *timeLimitAppender) AddFast(lset labels.Labels, ref uint64, t int64, v float64) error {
+	if t > app.maxTime {
+		return storage.ErrOutOfBounds
+	}
+	err := app.Appender.AddFast(lset, ref, t, v)
+	return err
 }
 
 // populateLabels builds a label set from the given label set and scrape configuration.
 // It returns a label set before relabeling was applied as the second return value.
-// Returns a nil label set if the target is dropped during relabeling.
-func populateLabels(lset model.LabelSet, cfg *config.ScrapeConfig) (res, orig model.LabelSet, err error) {
-	lset = lset.Clone()
-	// Copy labels into the labelset for the target if they are not
-	// set already. Apply the labelsets in order of decreasing precedence.
-	scrapeLabels := model.LabelSet{
-		model.SchemeLabel:      model.LabelValue(cfg.Scheme),
-		model.MetricsPathLabel: model.LabelValue(cfg.MetricsPath),
-		model.JobLabel:         model.LabelValue(cfg.JobName),
+// Returns the original discovered label set found before relabelling was applied if the target is dropped during relabeling.
+func populateLabels(lset labels.Labels, cfg *config.ScrapeConfig) (res, orig labels.Labels, err error) {
+	// Copy labels into the labelset for the target if they are not set already.
+	scrapeLabels := []labels.Label{
+		{Name: model.JobLabel, Value: cfg.JobName},
+		{Name: model.MetricsPathLabel, Value: cfg.MetricsPath},
+		{Name: model.SchemeLabel, Value: cfg.Scheme},
 	}
-	for ln, lv := range scrapeLabels {
-		if _, ok := lset[ln]; !ok {
-			lset[ln] = lv
+	lb := labels.NewBuilder(lset)
+
+	for _, l := range scrapeLabels {
+		if lv := lset.Get(l.Name); lv == "" {
+			lb.Set(l.Name, l.Value)
 		}
 	}
 	// Encode scrape query parameters as labels.
 	for k, v := range cfg.Params {
 		if len(v) > 0 {
-			lset[model.LabelName(model.ParamLabelPrefix+k)] = model.LabelValue(v[0])
+			lb.Set(model.ParamLabelPrefix+k, v[0])
 		}
 	}
 
-	preRelabelLabels := lset.Clone()
-	lset = relabel.Process(lset, cfg.RelabelConfigs...)
+	preRelabelLabels := lb.Labels()
+	lset = relabel.Process(preRelabelLabels, cfg.RelabelConfigs...)
 
 	// Check if the target was dropped.
 	if lset == nil {
-		return nil, nil, nil
+		return nil, preRelabelLabels, nil
 	}
-	if _, ok := lset[model.AddressLabel]; !ok {
+	if v := lset.Get(model.AddressLabel); v == "" {
 		return nil, nil, fmt.Errorf("no address")
 	}
 
+	lb = labels.NewBuilder(lset)
+
 	// addPort checks whether we should add a default port to the address.
 	// If the address is not valid, we don't append a port either.
 	addPort := func(s string) bool {
@@ -308,10 +299,11 @@ func populateLabels(lset model.LabelSet, cfg *config.ScrapeConfig) (res, orig mo
 		_, _, err := net.SplitHostPort(s + ":1234")
 		return err == nil
 	}
+	addr := lset.Get(model.AddressLabel)
 	// If it's an address with no trailing port, infer it based on the used scheme.
-	if addr := string(lset[model.AddressLabel]); addPort(addr) {
+	if addPort(addr) {
 		// Addresses reaching this point are already wrapped in [] if necessary.
-		switch lset[model.SchemeLabel] {
+		switch lset.Get(model.SchemeLabel) {
 		case "http", "":
 			addr = addr + ":80"
 		case "https":
@@ -319,49 +311,60 @@ func populateLabels(lset model.LabelSet, cfg *config.ScrapeConfig) (res, orig mo
 		default:
 			return nil, nil, fmt.Errorf("invalid scheme: %q", cfg.Scheme)
 		}
-		lset[model.AddressLabel] = model.LabelValue(addr)
+		lb.Set(model.AddressLabel, addr)
 	}
-	if err := config.CheckTargetAddress(lset[model.AddressLabel]); err != nil {
+
+	if err := config.CheckTargetAddress(model.LabelValue(addr)); err != nil {
 		return nil, nil, err
 	}
 
 	// Meta labels are deleted after relabelling. Other internal labels propagate to
 	// the target which decides whether they will be part of their label set.
-	for ln, lv := range lset {
-		if strings.HasPrefix(string(ln), model.MetaLabelPrefix) {
-			delete(lset, ln)
-			continue
-		}
-		// Check label values are valid, drop the target if not.
-		if !lv.IsValid() {
-			return nil, nil, fmt.Errorf("invalid label value for %q: %q", ln, lv)
+	for _, l := range lset {
+		if strings.HasPrefix(l.Name, model.MetaLabelPrefix) {
+			lb.Del(l.Name)
 		}
 	}
 
 	// Default the instance label to the target address.
-	if _, ok := lset[model.InstanceLabel]; !ok {
-		lset[model.InstanceLabel] = lset[model.AddressLabel]
+	if v := lset.Get(model.InstanceLabel); v == "" {
+		lb.Set(model.InstanceLabel, addr)
 	}
-	return lset, preRelabelLabels, nil
+
+	res = lb.Labels()
+	for _, l := range res {
+		// Check label values are valid, drop the target if not.
+		if !model.LabelValue(l.Value).IsValid() {
+			return nil, nil, fmt.Errorf("invalid label value for %q: %q", l.Name, l.Value)
+		}
+	}
+	return res, preRelabelLabels, nil
 }
 
 // targetsFromGroup builds targets based on the given TargetGroup and config.
-func targetsFromGroup(tg *config.TargetGroup, cfg *config.ScrapeConfig) ([]*Target, error) {
+func targetsFromGroup(tg *targetgroup.Group, cfg *config.ScrapeConfig) ([]*Target, error) {
 	targets := make([]*Target, 0, len(tg.Targets))
 
-	for i, lset := range tg.Targets {
-		// Combine target labels with target group labels.
+	for i, tlset := range tg.Targets {
+		lbls := make([]labels.Label, 0, len(tlset)+len(tg.Labels))
+
+		for ln, lv := range tlset {
+			lbls = append(lbls, labels.Label{Name: string(ln), Value: string(lv)})
+		}
 		for ln, lv := range tg.Labels {
-			if _, ok := lset[ln]; !ok {
-				lset[ln] = lv
+			if _, ok := tlset[ln]; !ok {
+				lbls = append(lbls, labels.Label{Name: string(ln), Value: string(lv)})
 			}
 		}
-		labels, origLabels, err := populateLabels(lset, cfg)
+
+		lset := labels.New(lbls...)
+
+		lbls, origLabels, err := populateLabels(lset, cfg)
 		if err != nil {
 			return nil, fmt.Errorf("instance %d in group %s: %s", i, tg, err)
 		}
-		if labels != nil {
-			targets = append(targets, NewTarget(labels, origLabels, cfg.Params))
+		if lbls != nil || origLabels != nil {
+			targets = append(targets, NewTarget(lbls, origLabels, cfg.Params))
 		}
 	}
 	return targets, nil
