@@ -15,6 +15,7 @@ package v1
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"errors"
 	"fmt"
@@ -23,19 +24,21 @@ import (
 	"net/http/httptest"
 	"net/url"
 	"reflect"
+	"strings"
 	"testing"
 	"time"
 
-	"github.com/golang/protobuf/proto"
+	"github.com/gogo/protobuf/proto"
 	"github.com/golang/snappy"
 	"github.com/prometheus/common/model"
 	"github.com/prometheus/common/route"
-	"golang.org/x/net/context"
 
 	"github.com/prometheus/prometheus/config"
+	"github.com/prometheus/prometheus/pkg/labels"
+	"github.com/prometheus/prometheus/pkg/timestamp"
+	"github.com/prometheus/prometheus/prompb"
 	"github.com/prometheus/prometheus/promql"
 	"github.com/prometheus/prometheus/retrieval"
-	"github.com/prometheus/prometheus/storage/metric"
 	"github.com/prometheus/prometheus/storage/remote"
 )
 
@@ -76,17 +79,17 @@ func TestEndpoints(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	now := model.Now()
+	now := time.Now()
 
 	tr := targetRetrieverFunc(func() []*retrieval.Target {
 		return []*retrieval.Target{
 			retrieval.NewTarget(
-				model.LabelSet{
+				labels.FromMap(map[string]string{
 					model.SchemeLabel:      "http",
 					model.AddressLabel:     "example.com:8080",
 					model.MetricsPathLabel: "/metrics",
-				},
-				model.LabelSet{},
+				}),
+				nil,
 				url.Values{},
 			),
 		}
@@ -101,18 +104,17 @@ func TestEndpoints(t *testing.T) {
 	})
 
 	api := &API{
-		Storage:               suite.Storage(),
+		Queryable:             suite.Storage(),
 		QueryEngine:           suite.QueryEngine(),
 		targetRetriever:       tr,
 		alertmanagerRetriever: ar,
-		now: func() model.Time { return now },
-		config: func() config.Config {
-			return samplePrometheusCfg
-		},
-		ready: func(f http.HandlerFunc) http.HandlerFunc { return f },
+		now:    func() time.Time { return now },
+		config: func() config.Config { return samplePrometheusCfg },
+		ready:  func(f http.HandlerFunc) http.HandlerFunc { return f },
 	}
 
-	start := model.Time(0)
+	start := time.Unix(0, 0)
+
 	var tests = []struct {
 		endpoint apiFunc
 		params   map[string]string
@@ -124,13 +126,13 @@ func TestEndpoints(t *testing.T) {
 			endpoint: api.query,
 			query: url.Values{
 				"query": []string{"2"},
-				"time":  []string{"123.3"},
+				"time":  []string{"123.4"},
 			},
 			response: &queryData{
-				ResultType: model.ValScalar,
-				Result: &model.Scalar{
-					Value:     2,
-					Timestamp: start.Add(123*time.Second + 300*time.Millisecond),
+				ResultType: promql.ValueTypeScalar,
+				Result: promql.Scalar{
+					V: 2,
+					T: timestamp.FromTime(start.Add(123*time.Second + 400*time.Millisecond)),
 				},
 			},
 		},
@@ -141,10 +143,10 @@ func TestEndpoints(t *testing.T) {
 				"time":  []string{"1970-01-01T00:02:03Z"},
 			},
 			response: &queryData{
-				ResultType: model.ValScalar,
-				Result: &model.Scalar{
-					Value:     0.333,
-					Timestamp: start.Add(123 * time.Second),
+				ResultType: promql.ValueTypeScalar,
+				Result: promql.Scalar{
+					V: 0.333,
+					T: timestamp.FromTime(start.Add(123 * time.Second)),
 				},
 			},
 		},
@@ -155,10 +157,10 @@ func TestEndpoints(t *testing.T) {
 				"time":  []string{"1970-01-01T01:02:03+01:00"},
 			},
 			response: &queryData{
-				ResultType: model.ValScalar,
-				Result: &model.Scalar{
-					Value:     0.333,
-					Timestamp: start.Add(123 * time.Second),
+				ResultType: promql.ValueTypeScalar,
+				Result: promql.Scalar{
+					V: 0.333,
+					T: timestamp.FromTime(start.Add(123 * time.Second)),
 				},
 			},
 		},
@@ -168,10 +170,10 @@ func TestEndpoints(t *testing.T) {
 				"query": []string{"0.333"},
 			},
 			response: &queryData{
-				ResultType: model.ValScalar,
-				Result: &model.Scalar{
-					Value:     0.333,
-					Timestamp: now,
+				ResultType: promql.ValueTypeScalar,
+				Result: promql.Scalar{
+					V: 0.333,
+					T: timestamp.FromTime(now),
 				},
 			},
 		},
@@ -184,15 +186,15 @@ func TestEndpoints(t *testing.T) {
 				"step":  []string{"1"},
 			},
 			response: &queryData{
-				ResultType: model.ValMatrix,
-				Result: model.Matrix{
-					&model.SampleStream{
-						Values: []model.SamplePair{
-							{Value: 0, Timestamp: start},
-							{Value: 1, Timestamp: start.Add(1 * time.Second)},
-							{Value: 2, Timestamp: start.Add(2 * time.Second)},
+				ResultType: promql.ValueTypeMatrix,
+				Result: promql.Matrix{
+					promql.Series{
+						Points: []promql.Point{
+							{V: 0, T: timestamp.FromTime(start)},
+							{V: 1, T: timestamp.FromTime(start.Add(1 * time.Second))},
+							{V: 2, T: timestamp.FromTime(start.Add(2 * time.Second))},
 						},
-						Metric: model.Metric{},
+						Metric: nil,
 					},
 				},
 			},
@@ -282,7 +284,7 @@ func TestEndpoints(t *testing.T) {
 			params: map[string]string{
 				"name": "__name__",
 			},
-			response: model.LabelValues{
+			response: []string{
 				"test_metric1",
 				"test_metric2",
 			},
@@ -292,7 +294,7 @@ func TestEndpoints(t *testing.T) {
 			params: map[string]string{
 				"name": "foo",
 			},
-			response: model.LabelValues{
+			response: []string{
 				"bar",
 				"boo",
 			},
@@ -310,11 +312,8 @@ func TestEndpoints(t *testing.T) {
 			query: url.Values{
 				"match[]": []string{`test_metric2`},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric2",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric2", "foo", "boo"),
 			},
 		},
 		{
@@ -322,23 +321,17 @@ func TestEndpoints(t *testing.T) {
 			query: url.Values{
 				"match[]": []string{`test_metric1{foo=~".+o"}`},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric1",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric1", "foo", "boo"),
 			},
 		},
 		{
 			endpoint: api.series,
 			query: url.Values{
-				"match[]": []string{`test_metric1{foo=~"o$"}`, `test_metric1{foo=~".+o"}`},
+				"match[]": []string{`test_metric1{foo=~".+o$"}`, `test_metric1{foo=~".+o"}`},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric1",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric1", "foo", "boo"),
 			},
 		},
 		{
@@ -346,11 +339,8 @@ func TestEndpoints(t *testing.T) {
 			query: url.Values{
 				"match[]": []string{`test_metric1{foo=~".+o"}`, `none`},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric1",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric1", "foo", "boo"),
 			},
 		},
 		// Start and end before series starts.
@@ -361,7 +351,7 @@ func TestEndpoints(t *testing.T) {
 				"start":   []string{"-2"},
 				"end":     []string{"-1"},
 			},
-			response: []model.Metric{},
+			response: []labels.Labels{},
 		},
 		// Start and end after series ends.
 		{
@@ -371,7 +361,7 @@ func TestEndpoints(t *testing.T) {
 				"start":   []string{"100000"},
 				"end":     []string{"100001"},
 			},
-			response: []model.Metric{},
+			response: []labels.Labels{},
 		},
 		// Start before series starts, end after series ends.
 		{
@@ -381,11 +371,8 @@ func TestEndpoints(t *testing.T) {
 				"start":   []string{"-1"},
 				"end":     []string{"100000"},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric2",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric2", "foo", "boo"),
 			},
 		},
 		// Start and end within series.
@@ -396,11 +383,8 @@ func TestEndpoints(t *testing.T) {
 				"start":   []string{"1"},
 				"end":     []string{"100"},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric2",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric2", "foo", "boo"),
 			},
 		},
 		// Start within series, end after.
@@ -411,11 +395,8 @@ func TestEndpoints(t *testing.T) {
 				"start":   []string{"1"},
 				"end":     []string{"100000"},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric2",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric2", "foo", "boo"),
 			},
 		},
 		// Start before series, end within series.
@@ -426,11 +407,8 @@ func TestEndpoints(t *testing.T) {
 				"start":   []string{"-1"},
 				"end":     []string{"1"},
 			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric2",
-					"foo":      "boo",
-				},
+			response: []labels.Labels{
+				labels.FromStrings("__name__", "test_metric2", "foo", "boo"),
 			},
 		},
 		// Missing match[] query params in series requests.
@@ -440,47 +418,15 @@ func TestEndpoints(t *testing.T) {
 		},
 		{
 			endpoint: api.dropSeries,
-			errType:  errorBadData,
-		},
-		// The following tests delete time series from the test storage. They
-		// must remain at the end and are fixed in their order.
-		{
-			endpoint: api.dropSeries,
-			query: url.Values{
-				"match[]": []string{`test_metric1{foo=~".+o"}`},
-			},
-			response: struct {
-				NumDeleted int `json:"numDeleted"`
-			}{1},
-		},
-		{
-			endpoint: api.series,
-			query: url.Values{
-				"match[]": []string{`test_metric1`},
-			},
-			response: []model.Metric{
-				{
-					"__name__": "test_metric1",
-					"foo":      "bar",
-				},
-			},
-		},
-		{
-			endpoint: api.dropSeries,
-			query: url.Values{
-				"match[]": []string{`{__name__=~".+"}`},
-			},
-			response: struct {
-				NumDeleted int `json:"numDeleted"`
-			}{2},
+			errType:  errorInternal,
 		},
 		{
 			endpoint: api.targets,
 			response: &TargetDiscovery{
 				ActiveTargets: []*Target{
 					{
-						DiscoveredLabels: model.LabelSet{},
-						Labels:           model.LabelSet{},
+						DiscoveredLabels: map[string]string{},
+						Labels:           map[string]string{},
 						ScrapeURL:        "http://example.com:8080/metrics",
 						Health:           "unknown",
 					},
@@ -505,35 +451,53 @@ func TestEndpoints(t *testing.T) {
 		},
 	}
 
-	for _, test := range tests {
-		// Build a context with the correct request params.
-		ctx := context.Background()
-		for p, v := range test.params {
-			ctx = route.WithParam(ctx, p, v)
+	methods := func(f apiFunc) []string {
+		fp := reflect.ValueOf(f).Pointer()
+		if fp == reflect.ValueOf(api.query).Pointer() || fp == reflect.ValueOf(api.queryRange).Pointer() {
+			return []string{http.MethodGet, http.MethodPost}
 		}
+		return []string{http.MethodGet}
+	}
 
-		req, err := http.NewRequest("ANY", fmt.Sprintf("http://example.com?%s", test.query.Encode()), nil)
-		if err != nil {
-			t.Fatal(err)
+	request := func(m string, q url.Values) (*http.Request, error) {
+		if m == http.MethodPost {
+			r, err := http.NewRequest(m, "http://example.com", strings.NewReader(q.Encode()))
+			r.Header.Set("Content-Type", "application/x-www-form-urlencoded")
+			return r, err
 		}
-		resp, apiErr := test.endpoint(req.WithContext(ctx))
-		if apiErr != nil {
-			if test.errType == errorNone {
-				t.Fatalf("Unexpected error: %s", apiErr)
+		return http.NewRequest(m, fmt.Sprintf("http://example.com?%s", q.Encode()), nil)
+	}
+
+	for _, test := range tests {
+		for _, method := range methods(test.endpoint) {
+			// Build a context with the correct request params.
+			ctx := context.Background()
+			for p, v := range test.params {
+				ctx = route.WithParam(ctx, p, v)
 			}
-			if test.errType != apiErr.typ {
-				t.Fatalf("Expected error of type %q but got type %q", test.errType, apiErr.typ)
+			t.Logf("run %s\t%q", method, test.query.Encode())
+
+			req, err := request(method, test.query)
+			if err != nil {
+				t.Fatal(err)
+			}
+			resp, apiErr := test.endpoint(req.WithContext(ctx))
+			if apiErr != nil {
+				if test.errType == errorNone {
+					t.Fatalf("Unexpected error: %s", apiErr)
+				}
+				if test.errType != apiErr.typ {
+					t.Fatalf("Expected error of type %q but got type %q", test.errType, apiErr.typ)
+				}
+				continue
+			}
+			if apiErr == nil && test.errType != errorNone {
+				t.Fatalf("Expected error of type %q but got none", test.errType)
+			}
+			if !reflect.DeepEqual(resp, test.response) {
+				t.Fatalf("Response does not match, expected:\n%+v\ngot:\n%+v", test.response, resp)
 			}
-			continue
 		}
-		if apiErr == nil && test.errType != errorNone {
-			t.Fatalf("Expected error of type %q but got none", test.errType)
-		}
-		if !reflect.DeepEqual(resp, test.response) {
-			t.Fatalf("Response does not match, expected:\n%+v\ngot:\n%+v", test.response, resp)
-		}
-		// Ensure that removed metrics are unindexed before the next request.
-		suite.Storage().WaitForIndexing()
 	}
 }
 
@@ -552,7 +516,7 @@ func TestReadEndpoint(t *testing.T) {
 	}
 
 	api := &API{
-		Storage:     suite.Storage(),
+		Queryable:   suite.Storage(),
 		QueryEngine: suite.QueryEngine(),
 		config: func() config.Config {
 			return config.Config{
@@ -568,19 +532,19 @@ func TestReadEndpoint(t *testing.T) {
 	}
 
 	// Encode the request.
-	matcher1, err := metric.NewLabelMatcher(metric.Equal, "__name__", "test_metric1")
+	matcher1, err := labels.NewMatcher(labels.MatchEqual, "__name__", "test_metric1")
 	if err != nil {
 		t.Fatal(err)
 	}
-	matcher2, err := metric.NewLabelMatcher(metric.Equal, "d", "e")
+	matcher2, err := labels.NewMatcher(labels.MatchEqual, "d", "e")
 	if err != nil {
 		t.Fatal(err)
 	}
-	query, err := remote.ToQuery(0, 1, metric.LabelMatchers{matcher1, matcher2})
+	query, err := remote.ToQuery(0, 1, []*labels.Matcher{matcher1, matcher2})
 	if err != nil {
 		t.Fatal(err)
 	}
-	req := &remote.ReadRequest{Queries: []*remote.Query{query}}
+	req := &prompb.ReadRequest{Queries: []*prompb.Query{query}}
 	data, err := proto.Marshal(req)
 	if err != nil {
 		t.Fatal(err)
@@ -603,7 +567,7 @@ func TestReadEndpoint(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	var resp remote.ReadResponse
+	var resp prompb.ReadResponse
 	err = proto.Unmarshal(uncompressed, &resp)
 	if err != nil {
 		t.Fatal(err)
@@ -613,14 +577,22 @@ func TestReadEndpoint(t *testing.T) {
 		t.Fatalf("Expected 1 result, got %d", len(resp.Results))
 	}
 
-	result := remote.FromQueryResult(resp.Results[0])
-	expected := &model.Matrix{
-		&model.SampleStream{
-			Metric: model.Metric{"__name__": "test_metric1", "b": "c", "d": "e", "baz": "qux", "foo": "bar"},
-			Values: []model.SamplePair{model.SamplePair{Value: 1, Timestamp: 0}},
+	result := resp.Results[0]
+	expected := &prompb.QueryResult{
+		Timeseries: []*prompb.TimeSeries{
+			{
+				Labels: []*prompb.Label{
+					{Name: "__name__", Value: "test_metric1"},
+					{Name: "b", Value: "c"},
+					{Name: "baz", Value: "qux"},
+					{Name: "d", Value: "e"},
+					{Name: "foo", Value: "bar"},
+				},
+				Samples: []*prompb.Sample{{Value: 1, Timestamp: 0}},
+			},
 		},
 	}
-	if !reflect.DeepEqual(&result, expected) {
+	if !reflect.DeepEqual(result, expected) {
 		t.Fatalf("Expected response \n%v\n but got \n%v\n", result, expected)
 	}
 }
@@ -721,14 +693,6 @@ func TestParseTime(t *testing.T) {
 		}, {
 			input: "30s",
 			fail:  true,
-		}, {
-			// Internal int64 overflow.
-			input: "-148966367200.372",
-			fail:  true,
-		}, {
-			// Internal int64 overflow.
-			input: "148966367200.372",
-			fail:  true,
 		}, {
 			input:  "123",
 			result: time.Unix(123, 0),
@@ -754,9 +718,8 @@ func TestParseTime(t *testing.T) {
 			t.Errorf("Expected error for %q but got none", test.input)
 			continue
 		}
-		res := model.TimeFromUnixNano(test.result.UnixNano())
-		if !test.fail && ts != res {
-			t.Errorf("Expected time %v for input %q but got %v", res, test.input, ts)
+		if !test.fail && !ts.Equal(test.result) {
+			t.Errorf("Expected time %v for input %q but got %v", test.result, test.input, ts)
 		}
 	}
 }
